#include "DepthLayerTreeEmbedding.hpp"

DepthLayerTreeEmbeddingImpl::DepthLayerTreeEmbeddingImpl(int maxDepth, int maxElemPerDepth, int embeddingsSize, int outEmbeddingsSize, LSTMImpl::LSTMOptions options) : maxDepth(maxDepth), maxElemPerDepth(maxElemPerDepth)
{
  for (int i = 0; i < maxDepth; i++)
    depthLstm.emplace_back(register_module(fmt::format("lstm_{}",i), LSTM(embeddingsSize, outEmbeddingsSize, options)));
}

torch::Tensor DepthLayerTreeEmbeddingImpl::forward(torch::Tensor input)
{
  auto context = input.narrow(1, firstInputIndex, getInputSize());

  std::vector<torch::Tensor> outputs;

  for (unsigned int i = 0; i < depthLstm.size(); i++)
    for (unsigned int j = 0; j < focusedBuffer.size()+focusedStack.size(); j++)
      outputs.emplace_back(depthLstm[i](input.narrow(1,i*(focusedBuffer.size()+focusedStack.size())*columns.size()*maxElemPerDepth + j*maxElemPerDepth, maxElemPerDepth)));

  return torch::cat(outputs, 1);
}

std::size_t DepthLayerTreeEmbeddingImpl::getOutputSize()
{
  std::size_t outputSize = 0;

  for (auto & lstm : depthLstm)
    outputSize += lstm->getOutputSize(maxElemPerDepth);

  return outputSize;
}

std::size_t DepthLayerTreeEmbeddingImpl::getInputSize()
{
  return (focusedBuffer.size()+focusedStack.size())*columns.size()*maxDepth*maxElemPerDepth;
}

void DepthLayerTreeEmbeddingImpl::addToContext(std::vector<std::vector<long>> & context, Dict & dict, const Config & config) const
{
  std::vector<long> focusedIndexes;

  for (int index : focusedBuffer)
    focusedIndexes.emplace_back(config.getRelativeWordIndex(index));

  for (int index : focusedStack)
    if (config.hasStack(index))
      focusedIndexes.emplace_back(config.getStack(index));
    else
      focusedIndexes.emplace_back(-1);

  for (auto & contextElement : context)
  {
    for (auto index : focusedIndexes)
    {

    }
  }
}

